from matplotlib import pyplot as plt
import numpy as np
import pandas as pd
import re
import scipy.optimize
import scipy.interpolate

params = {
'axes.labelsize': 8,
'font.size': 8,
'legend.fontsize': 9,
'xtick.labelsize': 9,
'ytick.labelsize': 9,
'text.usetex': False,
'figure.figsize': [6, 3.5],
'errorbar.capsize': 3,
'lines.linewidth': .8,
'font.family': 'CMU Serif',
'mathtext.fontset': 'cm'
}


plt.rcParams.update(params)


class Header:
    def parse(s):
        self = Header()
        m = re.match(r"([^,]*)\(([^\(]*)\)([^,])*", s.strip())

        net_name = m.group(1).strip()
        parameters = m.group(2).strip()
        axis = m.group(3).strip()

        parameters = { p.split("=")[0] : float(p.split("=")[1]) for p in re.findall(r"[^,]+", parameters)}

        self.parameters = parameters
        self.net_name = net_name
        self.axis = axis

        return self

    def __repr__(self):
        return self.net_name + " " + self.axis + repr(self.parameters) 


def import_data(s):
    with open(s) as f:
        header = f.readline()
    
        headers = list(map(Header.parse, re.findall(r"[^,]*\(.*?\)[^,]*", header)))
    
    df = pd.read_csv(s, skiprows=1, header=None, dtype=np.float64, na_values="  ")

    return headers, df

def plot_all(df, headers, filter={}, key_points=False, key=None, xoffset=0.0, lims=[9e-9, 44e-9]):
    high_start = []
    low = []
    high_end = []
    halftime_down = []
    halftime_up = []
    key_values = []


    for i, axis in list(enumerate(zip(headers[0::2], headers[1::2]))):
        x, y = axis
        assert(x.axis == "X")
        assert(y.axis == "Y")
    
        assert(x.parameters == y.parameters)

        skip = False
        for filter_name, filter_value in filter.items():
            if x.parameters[filter_name] != filter_value:
                skip = True

        if skip:
            continue
    
        if not key_points:
            plt.plot(df[2 * i] - xoffset, df[2 * i + 1], label=str(x.net_name), color="black")
        else:
            assert(key is not None)
            xdata = df[2 * i] - xoffset
            ydata = df[2 * i + 1]

            interpolation = scipy.interpolate.interp1d(xdata, ydata)
            key_values.append(x.parameters[key])
            high_start.append(interpolation(10e-9))
            high_end.append(interpolation(42e-9))
            low.append(interpolation(26e-9))
            
            try:
                halftime_down.append(scipy.optimize.brentq(lambda x: interpolation(x) - 0.9, 10e-9, 26e-9) - 10e-9)
            except:
                halftime_down.append(None)

            try:
                halftime_up.append(scipy.optimize.brentq(lambda x: interpolation(x) - 0.9, 26e-9, 42e-9) - 26e-9)
            except:
                halftime_up.append(None)


    if key_points:
        plt.figure()
        ax1 = plt.subplot()
        ax1.set_xlabel(key + ' [m]')
        ax1.set_ylabel('OUT [V]')
        ax1.plot(key_values, high_start, label="H_start", color="red")
        ax1.plot(key_values, high_end, label="H_end", color="blue")
        ax1.plot(key_values, low, label="L", color="black")
        ax1.legend(loc=0)
         
        ax2 = ax1.twinx()
        ax2.set_ylabel('time [s]')
        ax2.plot(key_values, halftime_down, label="T_down", color="black", linestyle=":")
        ax2.plot(key_values, halftime_up, label="T_up", color="black", linestyle="--")

        ax2.legend(loc=1)

    else:
        plt.xlim([lims[0] - xoffset, lims[1] - xoffset])

    

def process_trans(s, split_by=None, key="W"):
    headers, df = import_data(s)

    if split_by is not None:
        possible_values = set([h.parameters[split_by] for h in headers])

        for value in possible_values:
            # plot_all(df, headers, filter={split_by : value})
            plot_all(df, headers, filter={split_by : value}, key_points=True, key=key)
            plt.title(split_by + " = " + str(value))
            plt.ticklabel_format(style='sci', axis='x', scilimits=(0,0))
            plt.savefig(s[:-4] + "_" + split_by + "_"  + str(value) + ".pdf")
            plt.close()
    else:
        plot_all(df, headers, key_points=True, key=key)
        plt.ticklabel_format(style='sci', axis='x', scilimits=(0,0))
        plt.savefig(s[:-4] + "_key_" + str(key) + ".pdf")
        plt.close()


def process_dc(s, split_by=None):
    headers, df = import_data(s)

def process_sim(s, split_by=None, key="W"):
    trans_sim = s + "_trans.csv"
    dc_sim = s + "_dc.csv"

    process_trans(trans_sim, split_by, key=key)
    process_dc(dc_sim, split_by)

# in simulation order    
process_sim("w", split_by="PLUS")
process_sim("w_1.5u_mirror", split_by="PLUS")
process_sim("stage_1.5u_mirror_1.7u_diff", split_by="PLUS")
process_sim("diff_4u_mirror_1.7u_stage", split_by="PLUS")
process_sim("diff_4u_mirror_1.7u_stage_700n_isource", split_by="PLUS")
process_sim("mirror_1.7u_stage_700n_diff", split_by="PLUS")
process_sim("diff_2u_mirror_1.7u_stage_700n_isource_3u_mirror", split_by="PLUS")

def plot_trans(s, filter):
    headers, df = import_data(s + '_trans.csv')
    plt.figure()
    plot_all(df, headers, filter=filter, xoffset=1e-8)
    plt.xlabel("time [s]")
    plt.ylabel("OUT [V]")
   
    title = ""
    for key, value in filter.items():
        title += " and " + key + "= " + str(value)

    plt.title(title[4:])
    plt.legend()
    plt.savefig(s + "_trans_example.pdf")
    # plt.show()

plt.close()
plot_trans("w", filter={'W' : 1.031837e-06, 'PLUS': 0.016})
process_sim("diff_2u_mirror_880n_stage_700n_isource_3u_plus", key="PLUS")
plt.close()


def dc_ana(s, key_points=False, key=None, xoffset=0.0):
    headers, df = import_data(s + "_dc.csv")
    key_values = []
    offset = []

    for i, axis in list(enumerate(zip(headers[0::2], headers[1::2]))):
        x, y = axis
        assert(x.axis == "X")
        assert(y.axis == "Y")
    
        assert(x.parameters == y.parameters)
    
        if not key_points:
            plt.plot(df[2 * i] - xoffset, df[2 * i + 1], label=str(x.net_name), color="black")
        else:
            assert(key is not None)
            xdata = df[2 * i] - xoffset
            ydata = df[2 * i + 1]

            interpolation = scipy.interpolate.interp1d(xdata, ydata)
            key_values.append(x.parameters[key])
            
            try:
                offset.append(scipy.optimize.brentq(lambda x: interpolation(x) - 0.635, -0.008, 0.008))
            except:
                offset.append(None)


    if key_points:
        plt.figure()
        ax1 = plt.subplot()
        ax1.set_xlabel(key + ' [m]')
        ax1.set_ylabel('OUT [V]')
        ax1.plot(key_values, offset, label="offset", color="black")
        ax1.legend(loc=0)
    else:
        plt.xlim([9e-9 - xoffset, 44e-9 - xoffset])


    plt.ticklabel_format(style='sci', axis='x', scilimits=(0,0))
    plt.savefig(s[:-4] + "_key_dc_" + str(key) + ".pdf")
    # plt.show()


dc_ana("diff_2u_mirror_880n_stage_700n_isource_3u_plus", key="PLUS", key_points=True)
dc_ana("diff_2u_mirror_880n_stage_700n_isource_3u_idc_plus", key="PLUS", key_points=True)

data = np.loadtxt("dac.csv", skiprows=1, delimiter=",").T

x = data[0]
y = data[1]

plt.figure()
plt.plot(x, y, color="black")
plt.xlabel("time [ns]")
plt.ylabel("Vout [V]")
plt.ticklabel_format(style='sci', axis='x', scilimits=(0,0))
plt.savefig("dac_sim.pdf")
plt.close()


def plot_dc(s, filter):
    headers, df = import_data(s + '_dc.csv')
    plt.figure()
    plot_all(df, headers, filter=filter, lims=[-0.008, 0.008])
    plt.xlabel("Vminus [V]")
    plt.ylabel("OUT [V]")
   
    title = ""
    for key, value in filter.items():
        title += " and " + key + "= " + str(value)

    plt.title(title[4:])
    plt.legend()
    plt.savefig(s + "_dc_example.pdf")
    plt.show()

plot_dc("diff_2u_mirror_880n_stage_700n_isource_3u_plus", filter={'PLUS': 0.016})
